==> fop-0_20_0.tar.gz: fop/src/org/apache/fop/fo/pagination/RepeatablePageMasterAlternatives.java <==
/*
 * $Id$
 * Copyright (C) 2001 The Apache Software Foundation. All rights reserved.
 * For details on use and redistribution please refer to the
 * LICENSE file included with these sources.
 */

package org.apache.fop.fo.pagination;

import org.apache.fop.fo.*;
import org.apache.fop.apps.FOPException;
import org.apache.fop.messaging.MessageHandler;

// Java
import java.util.Vector;

public class RepeatablePageMasterAlternatives extends FObj
    implements SubSequenceSpecifier {

    private static final int INFINITE = -1;


    public static class Maker extends FObj.Maker {
        public FObj make(FObj parent,
                         PropertyList propertyList) throws FOPException {
            return new RepeatablePageMasterAlternatives(parent, propertyList);
        }

    }

    public static FObj.Maker maker() {
        return new RepeatablePageMasterAlternatives.Maker();
    }

    private PageSequenceMaster pageSequenceMaster;

    /**
     * Max times this page master can be repeated.
     * INFINITE is used for the unbounded case
     */
    private int maximumRepeats;
    private int numberConsumed = 0;

    private Vector conditionalPageMasterRefs;

    public RepeatablePageMasterAlternatives(FObj parent, PropertyList propertyList)
            throws FOPException {
        super(parent, propertyList);
        this.name = "fo:repeatable-page-master-alternatives";

        conditionalPageMasterRefs = new Vector();

        if (parent.getName().equals("fo:page-sequence-master")) {
            this.pageSequenceMaster = (PageSequenceMaster)parent;
            this.pageSequenceMaster.addSubsequenceSpecifier(this);
        } else {
            throw new FOPException("fo:repeatable-page-master-alternatives"
                                   + "must be child of fo:page-sequence-master, not "
                                   + parent.getName());
        }

        String mr = getProperty("maximum-repeats").getString();
        if (mr.equals("no-limit")) {
            setMaximumRepeats(INFINITE);
        } else {
            try {
                setMaximumRepeats(Integer.parseInt(mr));
            } catch (NumberFormatException nfe) {
                throw new FOPException("Invalid number for "
                                       + "'maximum-repeats' property");
            }
        }

    }

    public String getNextPageMaster(int currentPageNumber,
                                    boolean thisIsFirstPage,
                                    boolean isEmptyPage) {
        String pm = null;

        if (getMaximumRepeats() != INFINITE) {
            if (numberConsumed < getMaximumRepeats()) {
                numberConsumed++;
            } else {
                return null;
            }
        }

        for (int i = 0; i < conditionalPageMasterRefs.size(); i++) {
            ConditionalPageMasterReference cpmr =
                (ConditionalPageMasterReference)conditionalPageMasterRefs.elementAt(i);

            // 0-indexed page number
            if (cpmr.isValid(currentPageNumber + 1, thisIsFirstPage,
                             isEmptyPage)) {
                pm = cpmr.getMasterName();
                break;
            }
        }
        return pm;
    }

    private void setMaximumRepeats(int maximumRepeats) {
        if (maximumRepeats == INFINITE) {
            this.maximumRepeats = maximumRepeats;
        } else {
            this.maximumRepeats = (maximumRepeats < 0) ? 0 : maximumRepeats;
        }

    }

    private int getMaximumRepeats() {
        return this.maximumRepeats;
    }

    public void addConditionalPageMasterReference(ConditionalPageMasterReference cpmr) {
        this.conditionalPageMasterRefs.addElement(cpmr);
    }

    public void reset() {
        this.numberConsumed = 0;
    }


    protected PageSequenceMaster getPageSequenceMaster() {
        return pageSequenceMaster;
    }

}

==> fop-0_20_5rc2.tar.gz: fop/src/org/apache/fop/fo/pagination/RepeatablePageMasterAlternatives.java <==
/*
 * $Id$
 * Copyright (C) 2001 The Apache Software Foundation. All rights reserved.
 * For details on use and redistribution please refer to the
 * LICENSE file included with these sources.
 */

package org.apache.fop.fo.pagination;

import org.apache.fop.fo.*;
import org.apache.fop.apps.FOPException;

// Java
import java.util.ArrayList;

/**
 * Class modeling the fo:repeatable-page-master-alternatives object.
 *
 * @see <a href="@XSLFO-STD@#fo_repeatable-page-master-alternatives"
 *      target="_xslfostd">@XSLFO-STDID@
 *      &para;6.4.10</a>
 */
public class RepeatablePageMasterAlternatives extends FObj
    implements SubSequenceSpecifier {

    private static final int INFINITE = -1;


    public static class Maker extends FObj.Maker {
        public FObj make(FObj parent,
                         PropertyList propertyList) throws FOPException {
            return new RepeatablePageMasterAlternatives(parent, propertyList);
        }

    }

    public static FObj.Maker maker() {
        return new RepeatablePageMasterAlternatives.Maker();
    }

    private PageSequenceMaster pageSequenceMaster;

    /**
     * Max times this page master can be repeated.
     * INFINITE is used for the unbounded case
     */
    private int maximumRepeats;
    private int numberConsumed = 0;

    private ArrayList conditionalPageMasterRefs;

    public RepeatablePageMasterAlternatives(FObj parent, PropertyList propertyList)
            throws FOPException {
        super(parent, propertyList);

        if (parent.getName().equals("fo:page-sequence-master")) {
            this.pageSequenceMaster = (PageSequenceMaster)parent;
            this.pageSequenceMaster.addSubsequenceSpecifier(this);
        } else {
            throw new FOPException("A fo:repeatable-page-master-alternatives"
                                   + "must be child of fo:page-sequence-master, not "
                                   + parent.getName());
        }

        String mr = getProperty("maximum-repeats").getString();
        if (mr.equals("no-limit")) {
            this.maximumRepeats=INFINITE;
        } else {
            try {
                this.maximumRepeats = Integer.parseInt(mr);
                if (this.maximumRepeats < 0) {
                    log.debug("negative maximum-repeats: "+this.maximumRepeats);
                    this.maximumRepeats = 0;
                }
            } catch (NumberFormatException nfe) {
                throw new FOPException("Invalid number for "
                                       + "'maximum-repeats' property");
            }
        }
        conditionalPageMasterRefs = new ArrayList();
    }

    public String getName() {
        return "fo:repeatable-page-master-alternatives";
    }

    public void addConditionalPageMasterReference(ConditionalPageMasterReference cpmr) {
        this.conditionalPageMasterRefs.add(cpmr);
    }

    public String getNextPageMasterName(boolean isOddPage,
                                        boolean isFirstPage,
                                        boolean isEmptyPage) {
        if (maximumRepeats != -1) {
            if (numberConsumed < maximumRepeats) {
                numberConsumed++;
            } else {
                return null;
            }
        }

        for (int i = 0; i < conditionalPageMasterRefs.size(); i++) {
            ConditionalPageMasterReference cpmr =
                (ConditionalPageMasterReference)conditionalPageMasterRefs.get(i);
            if (cpmr.isValid(isOddPage, isFirstPage, isEmptyPage)) {
                return cpmr.getMasterName();
            }
        }
        return null;
    }

    public void reset() {
        this.numberConsumed = 0;
    }

}
